#!/usr/bin/python -OO
# Copyright 2005 Gregor Kaufmann <tdian@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""
sabnzbd.rss - rss client functionality
"""

__NAME__ = "RSS"


import os
import logging
import sabnzbd

from sabnzbd.misc import synchronized
from threading import RLock

try:
    import feedparser
    HAVE_FEEDPARSER = True
except ImportError:
    HAVE_FEEDPARSER = False
    
LOCK = RLock()
class RSSQueue:
    def __init__(self, uris= [], uri_table = {}, old_entries = {}):
        self.uris = uris
        self.uri_table = uri_table
        self.old_entries = old_entries
        
    @synchronized(LOCK)
    def run(self):
        for uri in self.uris:
            matches, match_table = self.uri_table[uri]
            
            logging.info("[%s] Parsing %s", __NAME__, uri)
            d = feedparser.parse(uri)
            logging.info("[%s] DOne parsing %s", __NAME__, uri)
            
            entries = d['entries']
            
            new = not uri in self.old_entries
            
            entry_links = []
            new_entry_links = []
            for entry in entries:
                link = entry['link']
                entry_links.append(link)
                if new or link not in self.old_entries[uri]:
                    new_entry_links.append(link)
                    
            logging.debug("[%s] new: %s", __NAME__, new)
            logging.debug("[%s] entry_links: %s", __NAME__, entry_links)
            logging.debug("[%s] new_entry_links: %s", __NAME__, 
                          new_entry_links)
                          
            self.old_entries[uri] = entry_links
            
            if new:
                continue
                
            for match in matches[:]:
                match_name, unpack_opts, match_multiple = match
                if not match_name in match_table:
                    match_table[match_name] = []
                    
                matched_links = match_table[match_name]
                
                logging.debug("[%s] %s matched_links: %s", __NAME__, match,
                              matched_links)
                              
                for link in matched_links[:]:
                    if link not in entry_links:
                        matched_links.remove(link)
                        logging.debug("[%s] Purged link (%s) removed", 
                                      __NAME__, link)
                                      
                for entry in entries:
                    link = entry['link']
                    
                    if link not in new_entry_links:
                        continue
                        
                    title = entry['title'].lower()
                    
                    found_match = True
                    if match_name != '*':
                        for match_part in match_name.split():
                            match_part = match_part.lower()
                            if match_part not in title:
                                found_match = False
                                break
                                
                    if found_match and link not in matched_links:
                        matched_links.append(link)
                        
                        _id = link_to_id(link)
                        if _id:
                            logging.info("[%s] Adding %s (%s) to queue", 
                                         __NAME__, _id, title)
                            sabnzbd.add_msgid(_id, unpack_opts)
                        else:
                            logging.info("[%s] Adding %s (%s) to queue", 
                                         __NAME__, link, title)
                            sabnzbd.add_url(link, unpack_opts)
                            
                        if not match_multiple:
                            matches.remove(match)
                            break
                            
        self.save()
        
    @synchronized(LOCK)
    def add_feed(self, uri, match_str):
        if uri not in self.uris:
            matches = []
            dupes = []
            match_str = match_str.split(':')
            for i in xrange(0, len(match_str), 3):
                match_name = match_str[i]
                unpack_opts = int(match_str[i+1])
                match_multiple = bool(int(match_str[i+2]))
                if match_name in dupes:
                    continue
                dupes.append(match_name)
                matches.append([match_name, unpack_opts, match_multiple])
                
            self.uris.append(uri)
            self.uri_table[uri] = [matches, {}]
            self.run()
            
    @synchronized(LOCK)
    def del_feed(self, uri_id):
        for uri in self.uris:
            if uri_id == str(id(uri)):
                self.uris.remove(uri)
                self.uri_table.pop(uri)
                if uri in self.old_entries:
                    self.old_entries.pop(uri)
                self.save()
                break
                
    @synchronized(LOCK)
    def get_info(self):
        return ([(uri, str(id(uri))) for uri in self.uris], self.uri_table.copy())
        
    @synchronized(LOCK)
    def save(self):
        sabnzbd.save_data((self.uris, self.uri_table, self.old_entries), 
                          sabnzbd.RSS_FILE_NAME)
                          
def link_to_id(link):
    _id = None
    
    if 'newzbin' in link.lower():
        if link[-1] == '/':
            _id = int(os.path.basename(link[:-1]))
        else:
            _id = int(os.path.basename(link))
            
    return _id
